@@ -32,16 +32,12 @@
 #include <QTextStream>
 #include <QStandardPaths>
 
-DiscoverMainWindow::CompactMode decodeCompactMode(const QString &str)
-{
-    if (str == QLatin1String("auto"))
-        return DiscoverMainWindow::Auto;
-    else if (str == QLatin1String("compact"))
-        return DiscoverMainWindow::Compact;
-    else if (str == QLatin1String("full"))
-        return DiscoverMainWindow::Full;
-    return DiscoverMainWindow::Full;
-}
+typedef QHash<QString, DiscoverMainWindow::CompactMode> StringCompactMode;
+Q_GLOBAL_STATIC_WITH_ARGS(StringCompactMode, s_decodeCompactMode, (StringCompactMode {
+    { QLatin1String("auto"), DiscoverMainWindow::Auto },
+    { QLatin1String("compact"), DiscoverMainWindow::Compact },
+    { QLatin1String("full"), DiscoverMainWindow::Full }
+}))
 
 QCommandLineParser* createParser()
 {
@@ -52,6 +48,7 @@ QCommandLineParser* createParser()
     parser->addOption(QCommandLineOption(QStringLiteral("mode"), i18n("Open Discover in a said mode. Modes correspond to the toolbar buttons."), QStringLiteral("name")));
     parser->addOption(QCommandLineOption(QStringLiteral("listmodes"), i18n("List all the available modes.")));
     parser->addOption(QCommandLineOption(QStringLiteral("compact"), i18n("Compact Mode (auto/compact/full)."), QStringLiteral("mode"), QStringLiteral("auto")));
+    parser->addOption(QCommandLineOption(QStringLiteral("local-filename"), i18n("Local package file to install"), QStringLiteral("package")));
     parser->addOption(QCommandLineOption(QStringLiteral("test"), QStringLiteral("Test file"), QStringLiteral("file.qml")));
     parser->addPositionalArgument(QStringLiteral("urls"), i18n("Supports appstream: url scheme"));
     DiscoverBackendsFactory::setupCommandLine(parser);
@@ -64,7 +61,7 @@ QCommandLineParser* createParser()
 void processArgs(QCommandLineParser* parser, DiscoverMainWindow* mainWindow)
 {
     if(parser->isSet(QStringLiteral("application")))
-        mainWindow->openApplication(parser->value(QStringLiteral("application")));
+        mainWindow->openApplication(QUrl(parser->value(QStringLiteral("application"))));
     else if(parser->isSet(QStringLiteral("mime")))
         mainWindow->openMimeType(parser->value(QStringLiteral("mime")));
     else if(parser->isSet(QStringLiteral("category")))
@@ -73,19 +70,11 @@ void processArgs(QCommandLineParser* parser, DiscoverMainWindow* mainWindow)
     if(parser->isSet(QStringLiteral("mode")))
         mainWindow->openMode(parser->value(QStringLiteral("mode")));
 
+    if(parser->isSet(QStringLiteral("local-filename")))
+        mainWindow->openLocalPackage(QUrl::fromUserInput(parser->value(QStringLiteral("local-filename")), {}, QUrl::AssumeLocalFile));
+
     foreach(const QString &arg, parser->positionalArguments()) {
-        QUrl url(arg);
-        if (url.scheme() == QLatin1String("appstream")) {
-            QTextStream(stdout) << "opening appstream resource" << url.host() << "from" << url.toString() << '\n';
-            if (url.host().isEmpty())
-                mainWindow->showPassiveNotification(i18n("Malformed url '%1'", url.toDisplayString()));
-            else
-                mainWindow->openApplication(url.host());
-        } else {
-            const QString msg = i18n("Unrecognized url: %1", url.toDisplayString());
-            QTextStream(stdout) << msg << '\n';
-            mainWindow->showPassiveNotification(msg);
-        }
+        mainWindow->openApplication(QUrl(arg));
     }
 }
 
@@ -117,8 +106,8 @@ int main(int argc, char** argv)
 
         KDBusService* service = new KDBusService(KDBusService::Unique, &app);
 
-        mainWindow = new DiscoverMainWindow(decodeCompactMode(parser->value(QStringLiteral("compact"))));
-        QObject::connect(&app, &QApplication::aboutToQuit, mainWindow, &DiscoverMainWindow::deleteLater);
+        mainWindow = new DiscoverMainWindow(s_decodeCompactMode->value(parser->value(QStringLiteral("compact")), DiscoverMainWindow::Full));
+        QObject::connect(&app, &QCoreApplication::aboutToQuit, mainWindow, &DiscoverMainWindow::deleteLater);
         QObject::connect(service, &KDBusService::activateRequested, mainWindow, [mainWindow](const QStringList &arguments, const QString &/*workingDirectory*/){
             mainWindow->rootObject()->raise();
             if (arguments.isEmpty())
