@@ -20,6 +20,7 @@
 
 #include "DummyReviewsBackend.h"
 #include "DummyBackend.h"
+#include "DummyResource.h"
 #include <ReviewsBackend/Review.h>
 #include <ReviewsBackend/Rating.h>
 #include <resources/AbstractResource.h>
@@ -50,12 +51,13 @@ Rating* DummyReviewsBackend::ratingForApplication(AbstractResource* app) const
 
 void DummyReviewsBackend::initialize()
 {
+    int i = 11;
     DummyBackend* b = qobject_cast<DummyBackend*>(parent());
-    foreach(AbstractResource* app, b->allResources()) {
+    foreach(DummyResource* app, b->resources()) {
         if (m_ratings.contains(app))
             continue;
         auto randomRating = qrand()%10;
-        Rating* rating = new Rating(app->packageName(), 15, randomRating, QStringLiteral("\"0, 0, 0, 4, %1\"").arg(randomRating));
+        Rating* rating = new Rating(app->packageName(), ++i, randomRating, QStringLiteral("\"0, 0, 0, 4, %1\"").arg(randomRating));
         rating->setParent(this);
         m_ratings.insert(app, rating);
     }
@@ -67,3 +69,8 @@ void DummyReviewsBackend::submitUsefulness(Review* r, bool useful)
     qDebug() << "usefulness..." << r->applicationName() << r->reviewer() << useful;
     r->setUsefulChoice(useful ? ReviewsModel::Yes : ReviewsModel::No);
 }
+
+void DummyReviewsBackend::submitReview(AbstractResource* res, const QString& a, const QString& b, const QString& c)
+{
+    qDebug() << "dummy submit review" << res->name() << a << b << c;
+}
