@@ -21,58 +21,67 @@
 #ifndef KNSBACKEND_H
 #define KNSBACKEND_H
 
-// KDE includes
-#include <KNewStuff3/kns3/entry.h>
+#include <KNSCore/EntryInternal>
 
-// DiscoverCommon includes
 #include <resources/AbstractResourcesBackend.h>
 #include "Transaction/AddonList.h"
-
 #include "discovercommon_export.h"
 
 class KConfigGroup;
 class KNSReviews;
+class KNSResource;
 class StandardBackendUpdater;
 
-namespace KNS3 { class DownloadManager; }
+namespace KNSCore { class Engine; }
 
 class DISCOVERCOMMON_EXPORT KNSBackend : public AbstractResourcesBackend
 {
 Q_OBJECT
 public:
-    explicit KNSBackend(QObject* parent = nullptr);
+    explicit KNSBackend(QObject* parent, const QString& iconName, const QString &knsrc);
     ~KNSBackend() override;
-    
-    void setMetaData(const QString& path) override;
+
     void removeApplication(AbstractResource* app) override;
     void installApplication(AbstractResource* app) override;
     void installApplication(AbstractResource* app, const AddonList& addons) override;
-    AbstractResource* resourceByPackageName(const QString& name) const override;
     int updatesCount() const override;
     AbstractReviewsBackend* reviewsBackend() const override;
-    QList<AbstractResource*> searchPackageName(const QString& searchText) override;
-    QVector< AbstractResource* > allResources() const override;
     AbstractBackendUpdater* backendUpdater() const override;
     bool isFetching() const override;
     QList<QAction*> messageActions() const override { return QList<QAction*>(); }
+    ResultsStream* search(const AbstractResourcesBackend::Filters & filter) override;
+    ResultsStream* findResourceByPackageName(const QUrl & search) override;
+
+    QVector<Category*> category() const override { return m_rootCategories; }
 
     bool isValid() const override;
 
-    QStringList extends() const { return m_extends; }
+    QStringList extends() const override { return m_extends; }
 
     QString iconName() const { return m_iconName; }
 
+    KNSCore::Engine* downloadManager() const { return m_engine; }
+
+Q_SIGNALS:
+    void receivedResources(const QVector<AbstractResource*> &resources);
+    void searchFinished();
+    void startingSearch();
+    void availableForQueries();
+
 public Q_SLOTS:
-    void receivedEntries(const KNS3::Entry::List& entries);
-    void statusChanged(const KNS3::Entry& entry);
+    void receivedEntries(const KNSCore::EntryInternal::List& entries);
+    void statusChanged(const KNSCore::EntryInternal& entry);
 
 private:
+    KNSResource* resourceForEntry(const KNSCore::EntryInternal& entry);
     void setFetching(bool f);
     void markInvalid(const QString &message);
+    ResultsStream* searchStream(const QString &searchText);
     
+    bool m_responsePending = false;
     bool m_fetching;
     bool m_isValid;
-    KNS3::DownloadManager* m_manager;
+    KNSCore::Engine* m_engine;
     QHash<QString, AbstractResource*> m_resourcesByName;
     int m_page;
     KNSReviews* const m_reviews;
@@ -80,6 +89,8 @@ private:
     QString m_iconName;
     StandardBackendUpdater* const m_updater;
     QStringList m_extends;
+    QStringList m_categories;
+    QVector<Category*> m_rootCategories;
 };
 
 #endif // KNSBACKEND_H
