@@ -25,6 +25,7 @@
 #include "PackageKitUpdater.h"
 #include "AppPackageKitResource.h"
 #include "PKTransaction.h"
+#include "LocalFilePKResource.h"
 #include "AppstreamReviews.h"
 #include <resources/AbstractResource.h>
 #include <resources/StandardBackendUpdater.h>
@@ -35,7 +36,6 @@
 #include <QStringList>
 #include <QDebug>
 #include <QTimer>
-#include <QTimerEvent>
 #include <QStandardPaths>
 
 #include <PackageKit/Transaction>
@@ -63,19 +63,13 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     , m_isFetching(0)
     , m_reviews(new AppstreamReviews(this))
 {
-#ifdef NEWAPPSTREAM
     bool b = m_appdata.load();
-#else
-    bool b = m_appdata.open();
-#endif
     if (!b) {
         qWarning() << "Could not open the AppStream metadata pool";
 
-        auto msg = new QAction(i18n("Got it"), this);
-        msg->setWhatsThis(i18n("Please make sure that Appstream is properly set up on your system"));
-        msg->setPriority(QAction::HighPriority);
-        connect(msg, &QAction::triggered, msg, [msg](){ msg->setVisible(false); });
-        m_messageActions << msg;
+        QTimer::singleShot(0, this, [this]() {
+            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
+        });
     }
     reloadPackageList();
 
@@ -159,37 +153,64 @@ void PackageKitBackend::reloadPackageList()
         disconnect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
     }
 
-#ifdef NEWAPPSTREAM
     const auto components = m_appdata.components();
-#else
-    const auto components = m_appdata.allComponents();
-#endif
     QStringList neededPackages;
     neededPackages.reserve(components.size());
-    foreach(const Appstream::Component& component, components) {
-        if (component.packageNames().isEmpty()) {
+    foreach(const AppStream::Component& component, components) {
+        const auto pkgNames = component.packageNames();
+        if (pkgNames.isEmpty()) {
+            if (component.kind() == AppStream::Component::KindDesktopApp) {
+                const QString file = locateService(component.desktopId());
+                if (!file.isEmpty()) {
+                    auto trans = PackageKit::Daemon::searchFiles(file);
+                    connect(trans, &PackageKit::Transaction::package, this, [trans](PackageKit::Transaction::Info info, const QString &packageID){
+                        if (info == PackageKit::Transaction::InfoInstalled)
+                            trans->setProperty("installedPackage", packageID);
+                    });
+                    connect(trans, &PackageKit::Transaction::finished, this, [this, trans, component](PackageKit::Transaction::Exit status) {
+                        const auto pkgidVal = trans->property("installedPackage");
+                        if (status == PackageKit::Transaction::ExitSuccess && !pkgidVal.isNull()) {
+                            const auto pkgid = pkgidVal.toString();
+                            acquireFetching(true);
+                            auto res = addComponent(component, {PackageKit::Daemon::packageName(pkgid)});
+                            res->addPackageId(PackageKit::Transaction::InfoInstalled, pkgid, true);
+                            acquireFetching(false);
+                        }
+                    });
+                    continue;
+                }
+            }
+
             qDebug() << "no packages for" << component.name();
             continue;
         }
-        neededPackages += component.packageNames();
+        neededPackages += pkgNames;
 
-        const auto res = new AppPackageKitResource(component, this);
-        m_packages.packages[component.id()] = res;
-        foreach (const QString& pkg, component.packageNames()) {
-            m_packages.packageToApp[pkg] += component.id();
-        }
-
-        foreach (const QString& pkg, component.extends()) {
-            m_packages.extendedBy[pkg] += res;
-        }
+        addComponent(component, pkgNames);
     }
+
     acquireFetching(false);
     neededPackages.removeDuplicates();
-
-    qDebug() << "needed..." << neededPackages.count();
     resolvePackages(neededPackages);
 }
 
+AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Component& component, const QStringList& pkgNames)
+{
+    Q_ASSERT(isFetching());
+    Q_ASSERT(!pkgNames.isEmpty());
+
+    const auto res = new AppPackageKitResource(component, pkgNames.at(0), this);
+    m_packages.packages[component.id()] = res;
+    foreach (const QString& pkg, pkgNames) {
+        m_packages.packageToApp[pkg] += component.id();
+    }
+
+    foreach (const QString& pkg, component.extends()) {
+        m_packages.extendedBy[pkg] += res;
+    }
+    return res;
+}
+
 class TransactionSet : public QObject
 {
     Q_OBJECT
@@ -303,7 +324,9 @@ void PackageKitBackend::includePackagesToAdd()
     foreach(PackageKitResource* res, m_packagesToDelete) {
         const auto pkgs = m_packages.packageToApp.value(res->packageName(), {res->packageName()});
         foreach(const auto &pkg, pkgs) {
-            m_packages.packages.take(pkg)->deleteLater();
+            auto res = m_packages.packages.take(pkg);
+            emit resourceRemoved(res);
+            res->deleteLater();
         }
     }
     m_packagesToAdd.clear();
@@ -314,6 +337,7 @@ void PackageKitBackend::includePackagesToAdd()
 void PackageKitBackend::transactionError(PackageKit::Transaction::Error, const QString& message)
 {
     qWarning() << "Transaction error: " << message << sender();
+    Q_EMIT passiveMessage(message);
 }
 
 void PackageKitBackend::packageDetails(const PackageKit::Details& details)
@@ -329,15 +353,7 @@ void PackageKitBackend::packageDetails(const PackageKit::Details& details)
 
 QSet<AbstractResource*> PackageKitBackend::resourcesByPackageName(const QString& name) const
 {
-    const QStringList names = m_packages.packageToApp.value(name, QStringList(name));
-    QSet<AbstractResource*> ret;
-    ret.reserve(names.size());
-    foreach(const QString& name, names) {
-        AbstractResource* res = m_packages.packages.value(name);
-        if (res)
-            ret += res;
-    }
-    return ret;
+    return resourcesByPackageNames<QSet<AbstractResource*>>({name});
 }
 
 template <typename T>
@@ -371,27 +387,49 @@ void PackageKitBackend::refreshDatabase()
     }
 }
 
-QVector<AbstractResource*> PackageKitBackend::allResources() const
+ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters& filter)
 {
-    return containerValues<QVector<AbstractResource*>>(m_packages.packages);
+    if (filter.search.isEmpty()) {
+        return new ResultsStream(QStringLiteral("PackageKitStream"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
+    } else {
+        const QList<AppStream::Component> components = m_appdata.search(filter.search);
+        const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
+        auto stream = new ResultsStream(QStringLiteral("PackageKitStream"));
+        if (!ids.isEmpty()) {
+            const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(ids);
+            QTimer::singleShot(0, this, [stream, resources, this] () {
+                stream->resourcesFound(resources);
+            });
+        }
+
+        PackageKit::Transaction * tArch = PackageKit::Daemon::resolve(filter.search, PackageKit::Transaction::FilterArch);
+        connect(tArch, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageArch);
+        connect(tArch, &PackageKit::Transaction::package, this, [tArch](PackageKit::Transaction::Info /*info*/, const QString &packageId){
+            tArch->setProperty("packageId", packageId);
+        });
+        connect(tArch, &PackageKit::Transaction::finished, this, [stream, tArch, ids, this]() {
+            getPackagesFinished();
+            const auto packageId = tArch->property("packageId");
+            if (!packageId.isNull()) {
+                const auto res = resourcesByPackageNames<QVector<AbstractResource*>>({PackageKit::Daemon::packageName(packageId.toString())});
+                stream->resourcesFound(kFilter<QVector<AbstractResource*>>(res, [ids](AbstractResource* res){ return !ids.contains(res->appstreamId()); }));
+            }
+            stream->deleteLater();
+        });
+        return stream;
+    }
 }
 
-AbstractResource* PackageKitBackend::resourceByPackageName(const QString& name) const
+ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
 {
-    const QStringList ids = m_packages.packageToApp.value(name, QStringList(name));
-    return ids.isEmpty() ? nullptr : m_packages.packages[ids.first()];
-}
-
-QList<AbstractResource*> PackageKitBackend::searchPackageName(const QString& searchText)
-{
-#ifdef NEWAPPSTREAM
-    const QList<Appstream::Component> components = m_appdata.search(searchText);
-#else
-    const QList<Appstream::Component> components = m_appdata.findComponentsByString(searchText, {});
-#endif
-    const QStringList ids = kTransform<QStringList>(components, [](const Appstream::Component& comp) { return comp.id(); });
-
-    return resourcesByPackageNames<QList<AbstractResource*>>(ids);
+    AbstractResource* pkg = nullptr;
+    if (url.scheme() == QLatin1String("appstream")) {
+        if (url.host().isEmpty())
+            passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
+        else
+            pkg = m_packages.packages.value(url.host());
+    }
+    return new ResultsStream(QStringLiteral("PackageKitStream"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
 }
 
 int PackageKitBackend::updatesCount() const
@@ -399,31 +437,6 @@ int PackageKitBackend::updatesCount() const
     return m_updatesPackageId.count();
 }
 
-void PackageKitBackend::transactionCanceled(Transaction* t)
-{
-    qDebug() << "Cancel transaction:" << t->resource()->packageName() << "with" << m_transactions.size() << "transactions running";
-    int count = m_transactions.removeAll(t);
-    Q_ASSERT(count==1);
-    Q_UNUSED(count)
-    TransactionModel::global()->cancelTransaction(t);
-}
-
-void PackageKitBackend::removeTransaction(Transaction* t)
-{
-    qDebug() << "Remove transaction:" << t->resource()->packageName() << "with" << m_transactions.size() << "transactions running";
-    int count = m_transactions.removeAll(t);
-    Q_ASSERT(count==1);
-    Q_UNUSED(count)
-    TransactionModel::global()->removeTransaction(t);
-}
-
-void PackageKitBackend::addTransaction(PKTransaction* t)
-{
-    m_transactions.append(t);
-    TransactionModel::global()->addTransaction(t);
-    t->start();
-}
-
 void PackageKitBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
     if(!addons.addonsToInstall().isEmpty())
@@ -437,27 +450,27 @@ void PackageKitBackend::installApplication(AbstractResource* app, const AddonLis
             appsToInstall += m_packages.packages.value(toInstall);
             Q_ASSERT(appsToInstall.last());
         }
-        addTransaction(new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole));
+        new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole);
     }
 
     if (!addons.addonsToRemove().isEmpty()) {
-        QVector<AbstractResource*> appsToRemove;
-        foreach(const QString& toRemove, addons.addonsToRemove()) {
-            appsToRemove += m_packages.packages.value(toRemove);
-        }
-        addTransaction(new PKTransaction(appsToRemove, Transaction::RemoveRole));
+        QVector<AbstractResource*> appsToRemove = kTransform<QVector<AbstractResource*>>(addons.addonsToRemove(), [this](const QString& toRemove){ return m_packages.packages.value(toRemove); });
+        new PKTransaction(appsToRemove, Transaction::RemoveRole);
     }
+
+    if (!app->isInstalled())
+        installApplication(app);
 }
 
 void PackageKitBackend::installApplication(AbstractResource* app)
 {
-    addTransaction(new PKTransaction({app}, Transaction::InstallRole));
+    new PKTransaction({app}, Transaction::InstallRole);
 }
 
 void PackageKitBackend::removeApplication(AbstractResource* app)
 {
     Q_ASSERT(!isFetching());
-    addTransaction(new PKTransaction({app}, Transaction::RemoveRole));
+    new PKTransaction({app}, Transaction::RemoveRole);
 }
 
 QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
@@ -564,4 +577,10 @@ AbstractReviewsBackend* PackageKitBackend::reviewsBackend() const
     return m_reviews;
 }
 
+AbstractResource * PackageKitBackend::resourceForFile(const QUrl& file)
+{
+    return new LocalFilePKResource(file, this);
+}
+
+
 #include "PackageKitBackend.moc"
