@@ -25,6 +25,7 @@
 #include <QUrl>
 #include <QStringList>
 #include <QScopedPointer>
+#include <QVector>
 #include <QCollatorSortKey>
 
 #include "discovercommon_export.h"
@@ -203,4 +204,6 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
         QScopedPointer<QCollatorSortKey> m_collatorKey;
 };
 
+Q_DECLARE_METATYPE(QVector<AbstractResource*>);
+
 #endif // ABSTRACTRESOURCE_H
