@@ -21,6 +21,10 @@
 #ifndef UTILS_H
 #define UTILS_H
 
+#include <functional>
+#include <QString>
+#include <QElapsedTimer>
+
 template <typename T, typename Q, typename _UnaryOperation>
 static T kTransform(const Q &input, _UnaryOperation op)
 {
@@ -32,4 +36,25 @@ static T kTransform(const Q &input, _UnaryOperation op)
     return ret;
 }
 
+template <typename T, typename Q, typename _UnaryOperation>
+static T kFilter(const Q &input, _UnaryOperation op)
+{
+    T ret;
+    for(const auto& v : input) {
+        if (op(v))
+            ret += v;
+    }
+    return ret;
+}
+
+class ElapsedDebug : private QElapsedTimer
+{
+public:
+    ElapsedDebug(const QString &name = QStringLiteral("<unnamed>")) : m_name(name) { start(); }
+    ~ElapsedDebug() { qDebug("elapsed %s: %lld!", m_name.toUtf8().constData(), elapsed()); }
+    void step(const QString &step) { qDebug("step(%s) %s: %lld!", qPrintable(step), m_name.toUtf8().constData(), elapsed()); }
+
+    QString m_name;
+};
+
 #endif
